﻿using System;
using System.Collections.Generic;
using System.Text;

namespace SocketCommon
{
    /// <summary>
    /// 文本信息
    /// </summary>
    public class MessageInfo
    {
        public string Content { get; set; }//信息内容
        public MessageInfo(string content)
        {
            this.Content = content;
        }
        public override string ToString()
        {
            return String.Format("<message Content=\"{0}\" />", this.Content);
        }
    }

    /// <summary>
    /// 文件信息
    /// </summary>
    public class RequestFile
    {
        public string Address { get; set; }//发送端Ip
        public int Port { get; set; }//端口号
        public RequestMode Mode { get; set; }//请求类
        public FileObject FileObject { get; set; }//文件详细参数
        public RequestFile() { }
        public RequestFile(RequestMode mode, FileObject fileobject)
        {
            this.Mode = mode;
            this.FileObject = fileobject;
        }
        public RequestFile(string address, int port, RequestMode mode, FileObject fileobject)
        {
            this.Address = address;
            this.Port = port;
            this.Mode = mode;
            this.FileObject = fileobject;
        }
        public RequestFile(string address, int port, RequestMode mode, string filename, long filelength, int packetsize, long packetcount)
        {
            this.Address = address;
            this.Port = port;
            this.Mode = mode;
            this.FileObject = new FileObject(filename, filelength, packetsize, packetcount);
        }
        public override string ToString()
        {
            StringBuilder sbString = new StringBuilder();
            sbString.Append("<message ");
            sbString.Append(String.Format("Address=\"{0}\" ", Address));
            sbString.Append(String.Format("Port=\"{0}\" ", Port));
            sbString.Append(String.Format("Mode=\"{0}\" ", Mode));
            sbString.Append(String.Format("FileName=\"{0}\" ", FileObject.FileName));
            sbString.Append(String.Format("FileLength=\"{0}\" ", FileObject.FileLength));
            sbString.Append(String.Format("PacketSize=\"{0}\" ", FileObject.PacketSize));
            sbString.Append(String.Format("PacketCount=\"{0}\" ", FileObject.PacketCount));
            sbString.Append("/>");
            return sbString.ToString();
        }
    }
    /// <summary>
    /// 订立信息协议
    /// </summary>
    public class MessageProtocol
    {
        public MessageType MessageType { get; set; }
        public MessageInfo MessageInfo { get; set; }
        public RequestFile RequestFile { get; set; }
        public MessageProtocol() { }
        public MessageProtocol(string msg)
        {
            MessageType = MessageType.text;
            MessageInfo = new MessageInfo(msg);
        }
        public MessageProtocol(RequestMode mode, FileObject fileobject)
        {
            MessageType = MessageType.file;
            RequestFile = new RequestFile(mode, fileobject);
        }
        public MessageProtocol(string address, int port, RequestMode mode, FileObject fileobject)
        {
            MessageType = MessageType.file;
            RequestFile = new RequestFile(address, port, mode, fileobject);
        }
        public override string ToString()
        {
            StringBuilder sbString = new StringBuilder();
            sbString.Append(String.Format("<protocol Type=\"{0}\">", MessageType));
            if (MessageType == MessageType.text)
            {
                sbString.Append(MessageInfo.ToString());
            }
            else
            {
                sbString.Append(RequestFile.ToString());
            }
            sbString.Append("</protocol>");
            return sbString.ToString();
        }
        public byte[] ToBytes()
        {
            return Encoding.UTF8.GetBytes(this.ToString());
        }
    }
}
